@@ -1,7 +1,10 @@
-from contextlib import contextmanager
+import functools
 import sys
 import traceback
+from contextlib import contextmanager
+
 import pytest
+from pytestqt.utils import get_marker
 
 
 @contextmanager
@@ -19,6 +22,12 @@ def capture_exceptions():
         manager.finish()
 
 
+def _except_hook(type_, value, tback, exceptions=None):
+    """Hook functions installed by _QtExceptionCaptureManager"""
+    exceptions.append((type_, value, tback))
+    sys.stderr.write(format_captured_exceptions([(type_, value, tback)]))
+
+
 class _QtExceptionCaptureManager(object):
     """
     Manages exception capture context.
@@ -32,12 +41,8 @@ class _QtExceptionCaptureManager(object):
         """Start exception capturing by installing a hook into sys.excepthook
         that records exceptions received into ``self.exceptions``.
         """
-        def hook(type_, value, tback):
-            self.exceptions.append((type_, value, tback))
-            sys.stderr.write(format_captured_exceptions([(type_, value, tback)]))
-
         self.old_hook = sys.excepthook
-        sys.excepthook = hook
+        sys.excepthook = functools.partial(_except_hook, exceptions=self.exceptions)
 
     def finish(self):
         """Stop exception capturing, restoring the original hook.
@@ -57,9 +62,10 @@ class _QtExceptionCaptureManager(object):
             self.finish()
             exceptions = self.exceptions
             self.exceptions = []
-            prefix = '%s ERROR: ' % when
-            pytest.fail(prefix + format_captured_exceptions(exceptions),
-                        pytrace=False)
+            prefix = "%s ERROR: " % when
+            msg = prefix + format_captured_exceptions(exceptions)
+            del exceptions[:]  # Don't keep exceptions alive longer.
+            pytest.fail(msg, pytrace=False)
 
 
 def format_captured_exceptions(exceptions):
@@ -67,19 +73,27 @@ def format_captured_exceptions(exceptions):
     Formats exceptions given as (type, value, traceback) into a string
     suitable to display as a test failure.
     """
-    message = 'Qt exceptions in virtual methods:\n'
-    message += '_' * 80 + '\n'
+    if sys.version_info.major == 2:
+        from StringIO import StringIO
+    else:
+        from io import StringIO
+
+    stream = StringIO()
+    stream.write("Qt exceptions in virtual methods:\n")
+    sep = "_" * 80 + "\n"
+    stream.write(sep)
     for (exc_type, value, tback) in exceptions:
-        message += ''.join(traceback.format_exception(exc_type, value, tback)) + '\n'
-        message += '_' * 80 + '\n'
-    return message
+        traceback.print_exception(exc_type, value, tback, file=stream)
+        stream.write(sep)
+    return stream.getvalue()
 
 
 def _is_exception_capture_enabled(item):
     """returns if exception capture is disabled for the given test item.
     """
-    disabled = item.get_marker('qt_no_exception_capture') or \
-               item.config.getini('qt_no_exception_capture')
+    disabled = get_marker(item, "qt_no_exception_capture") or item.config.getini(
+        "qt_no_exception_capture"
+    )
     return not disabled
 
 
@@ -93,6 +107,7 @@ class TimeoutError(Exception):
         In versions prior to ``2.1``, this exception was called ``SignalTimeoutError``.
         An alias is kept for backward compatibility.
     """
+
     pass
 
 
