@@ -4,16 +4,23 @@ import weakref
 
 from pytestqt.exceptions import SignalTimeoutError, TimeoutError
 from pytestqt.qt_compat import qt_api
-from pytestqt.wait_signal import SignalBlocker, MultiSignalBlocker, SignalEmittedSpy, SignalEmittedError
+from pytestqt.wait_signal import (
+    SignalBlocker,
+    MultiSignalBlocker,
+    SignalEmittedSpy,
+    SignalEmittedError,
+    CallbackBlocker,
+    CallbackCalledTwiceError,
+)
 
 
 def _parse_ini_boolean(value):
     if value in (True, False):
         return value
     try:
-        return {'true': True, 'false': False}[value.lower()]
+        return {"true": True, "false": False}[value.lower()]
     except KeyError:
-        raise ValueError('unknown string for bool: %r' % value)
+        raise ValueError("unknown string for bool: %r" % value)
 
 
 class QtBot(object):
@@ -96,7 +103,6 @@ class QtBot(object):
 
     .. staticmethod:: mouseClick (widget, button[, stateKey=0[, pos=QPoint()[, delay=-1]]])
     .. staticmethod:: mouseDClick (widget, button[, stateKey=0[, pos=QPoint()[, delay=-1]]])
-    .. staticmethod:: mouseEvent (action, widget, button, stateKey, pos[, delay=-1])
     .. staticmethod:: mouseMove (widget[, pos=QPoint()[, delay=-1]])
     .. staticmethod:: mousePress (widget, button[, stateKey=0[, pos=QPoint()[, delay=-1]]])
     .. staticmethod:: mouseRelease (widget, button[, stateKey=0[, pos=QPoint()[, delay=-1]]])
@@ -108,8 +114,10 @@ class QtBot(object):
         :param Qt.MouseButton button: flags OR'ed together representing the button pressed.
             Possible flags are:
 
-            * ``Qt.NoButton``: The button state does not refer to any button (see QMouseEvent.button()).
-            * ``Qt.LeftButton``: The left button is pressed, or an event refers to the left button. (The left button may be the right button on left-handed mice.)
+            * ``Qt.NoButton``: The button state does not refer to any button
+              (see QMouseEvent.button()).
+            * ``Qt.LeftButton``: The left button is pressed, or an event refers to the left button.
+              (The left button may be the right button on left-handed mice.)
             * ``Qt.RightButton``: The right button.
             * ``Qt.MidButton``: The middle button.
             * ``Qt.MiddleButton``: The middle button.
@@ -131,6 +139,19 @@ class QtBot(object):
     def __init__(self, request):
         self._request = request
 
+    def _should_raise(self, raising_arg):
+        ini_val = self._request.config.getini("qt_default_raising")
+        legacy_ini_val = self._request.config.getini("qt_wait_signal_raising")
+
+        if raising_arg is not None:
+            return raising_arg
+        elif legacy_ini_val:
+            return _parse_ini_boolean(legacy_ini_val)
+        elif ini_val:
+            return _parse_ini_boolean(ini_val)
+        else:
+            return True
+
     def addWidget(self, widget):
         """
         Adds a widget to be tracked by this bot. This is not required, but will ensure that the
@@ -171,7 +192,9 @@ class QtBot(object):
         .. note:: This method is also available as ``wait_active`` (pep-8 alias)
         """
         __tracebackhide__ = True
-        return _WaitWidgetContextManager('qWaitForWindowActive', 'activated', widget, timeout)
+        return _WaitWidgetContextManager(
+            "qWaitForWindowActive", "activated", widget, timeout
+        )
 
     wait_active = waitActive  # pep-8 alias
 
@@ -201,7 +224,9 @@ class QtBot(object):
         .. note:: This method is also available as ``wait_exposed`` (pep-8 alias)
         """
         __tracebackhide__ = True
-        return _WaitWidgetContextManager('qWaitForWindowExposed', 'exposed', widget, timeout)
+        return _WaitWidgetContextManager(
+            "qWaitForWindowExposed", "exposed", widget, timeout
+        )
 
     wait_exposed = waitExposed  # pep-8 alias
 
@@ -218,7 +243,7 @@ class QtBot(object):
 
         .. note:: This method is also available as ``wait_for_window_shown`` (pep-8 alias)
         """
-        if hasattr(qt_api.QtTest.QTest, 'qWaitForWindowExposed'):
+        if hasattr(qt_api.QtTest.QTest, "qWaitForWindowExposed"):
             return qt_api.QtTest.QTest.qWaitForWindowExposed(widget)
         else:
             return qt_api.QtTest.QTest.qWaitForWindowShown(widget)
@@ -288,7 +313,7 @@ class QtBot(object):
         :param bool raising:
             If :class:`QtBot.TimeoutError <pytestqt.plugin.TimeoutError>`
             should be raised if a timeout occurred.
-            This defaults to ``True`` unless ``qt_wait_signal_raising = false``
+            This defaults to ``True`` unless ``qt_default_raising = false``
             is set in the config.
         :param Callable check_params_cb:
             Optional ``callable`` that compares the provided signal parameters to some expected parameters.
@@ -304,20 +329,24 @@ class QtBot(object):
         .. note::
             This method is also available as ``wait_signal`` (pep-8 alias)
         """
-        if raising is None:
-            raising_val = self._request.config.getini('qt_wait_signal_raising')
-            if not raising_val:
-                raising = True
-            else:
-                raising = _parse_ini_boolean(raising_val)
-        blocker = SignalBlocker(timeout=timeout, raising=raising, check_params_cb=check_params_cb)
+        raising = self._should_raise(raising)
+        blocker = SignalBlocker(
+            timeout=timeout, raising=raising, check_params_cb=check_params_cb
+        )
         if signal is not None:
             blocker.connect(signal)
         return blocker
 
     wait_signal = waitSignal  # pep-8 alias
 
-    def waitSignals(self, signals=None, timeout=1000, raising=None, check_params_cbs=None, order="none"):
+    def waitSignals(
+        self,
+        signals=None,
+        timeout=1000,
+        raising=None,
+        check_params_cbs=None,
+        order="none",
+    ):
         """
         .. versionadded:: 1.4
 
@@ -348,7 +377,7 @@ class QtBot(object):
         :param bool raising:
             If :class:`QtBot.TimeoutError <pytestqt.plugin.TimeoutError>`
             should be raised if a timeout occurred.
-            This defaults to ``True`` unless ``qt_wait_signal_raising = false``
+            This defaults to ``True`` unless ``qt_default_raising = false``
             is set in the config.
         :param list check_params_cbs:
             optional list of callables that compare the provided signal parameters to some expected parameters.
@@ -380,14 +409,22 @@ class QtBot(object):
         if order not in ["none", "simple", "strict"]:
             raise ValueError("order has to be set to 'none', 'simple' or 'strict'")
 
-        if raising is None:
-            raising = self._request.config.getini('qt_wait_signal_raising')
+        raising = self._should_raise(raising)
 
         if check_params_cbs:
             if len(check_params_cbs) != len(signals):
-                raise ValueError("Number of callbacks ({}) does not "
-                                 "match number of signals ({})!".format(len(check_params_cbs), len(signals)))
-        blocker = MultiSignalBlocker(timeout=timeout, raising=raising, order=order, check_params_cbs=check_params_cbs)
+                raise ValueError(
+                    "Number of callbacks ({}) does not "
+                    "match number of signals ({})!".format(
+                        len(check_params_cbs), len(signals)
+                    )
+                )
+        blocker = MultiSignalBlocker(
+            timeout=timeout,
+            raising=raising,
+            order=order,
+            check_params_cbs=check_params_cbs,
+        )
         if signals is not None:
             blocker.add_signals(signals)
         return blocker
@@ -407,19 +444,24 @@ class QtBot(object):
         blocker.wait()
 
     @contextlib.contextmanager
-    def assertNotEmitted(self, signal):
+    def assertNotEmitted(self, signal, wait=0):
         """
         .. versionadded:: 1.11
 
         Make sure the given ``signal`` doesn't get emitted.
 
+        :param int wait:
+            How many milliseconds to wait to make sure the signal isn't emitted
+            asynchronously. By default, this method returns immediately and only
+            catches signals emitted inside the ``with``-block.
+
         This is intended to be used as a context manager.
 
         .. note:: This method is also available as ``assert_not_emitted``
                   (pep-8 alias)
         """
         spy = SignalEmittedSpy(signal)
-        with spy:
+        with spy, self.waitSignal(signal, timeout=wait, raising=False):
             yield
         spy.assert_not_emitted()
 
@@ -462,6 +504,7 @@ class QtBot(object):
         """
         __tracebackhide__ = True
         import time
+
         start = time.time()
 
         def timed_out():
@@ -477,7 +520,7 @@ class QtBot(object):
                     raise
             else:
                 if result not in (None, True, False):
-                    msg = 'waitUntil() callback must return None, True or False, returned %r'
+                    msg = "waitUntil() callback must return None, True or False, returned %r"
                     raise ValueError(msg % result)
 
                 # 'assert' form
@@ -488,11 +531,56 @@ class QtBot(object):
                 if result:
                     return
                 else:
-                    assert not timed_out(), 'waitUntil timed out in %s miliseconds' % timeout
+                    assert not timed_out(), (
+                        "waitUntil timed out in %s miliseconds" % timeout
+                    )
             self.wait(10)
 
     wait_until = waitUntil  # pep-8 alias
 
+    def waitCallback(self, timeout=1000, raising=None):
+        """
+        .. versionadded:: 3.1
+
+        Stops current test until a callback is called.
+
+        Used to stop the control flow of a test until the returned callback is
+        called, or a number of milliseconds, specified by ``timeout``, has
+        elapsed.
+
+        Best used as a context manager::
+
+           with qtbot.waitCallback() as callback:
+               function_taking_a_callback(callback)
+           assert callback.args == [True]
+
+        Also, you can use the :class:`CallbackBlocker` directly if the
+        context manager form is not convenient::
+
+           blocker = qtbot.waitCallback(timeout=1000)
+           function_calling_a_callback(blocker)
+           blocker.wait()
+
+
+        :param int timeout:
+            How many milliseconds to wait before resuming control flow.
+        :param bool raising:
+            If :class:`QtBot.TimeoutError <pytestqt.plugin.TimeoutError>`
+            should be raised if a timeout occurred.
+            This defaults to ``True`` unless ``qt_default_raising = false``
+            is set in the config.
+        :returns:
+            A ``CallbackBlocker`` object which can be used directly as a
+            callback as it implements ``__call__``.
+
+        .. note:: This method is also available as ``wait_callback`` (pep-8 alias)
+        """
+        raising = self._should_raise(raising)
+        blocker = CallbackBlocker(timeout=timeout, raising=raising)
+        return blocker
+
+    wait_callback = waitCallback  # pep-8 alias
+
     @contextlib.contextmanager
     def captureExceptions(self):
         """
@@ -512,6 +600,7 @@ class QtBot(object):
         .. note:: This method is also available as ``capture_exceptions`` (pep-8 alias)
         """
         from pytestqt.exceptions import capture_exceptions
+
         with capture_exceptions() as exceptions:
             yield exceptions
 
@@ -539,20 +628,18 @@ class QtBot(object):
 
         # inject methods from QTest into QtBot
         method_names = [
-            'keyPress',
-            'keyClick',
-            'keyClicks',
-            'keyEvent',
-            'keyPress',
-            'keyRelease',
-            'keyToAscii',
-
-            'mouseClick',
-            'mouseDClick',
-            'mouseEvent',
-            'mouseMove',
-            'mousePress',
-            'mouseRelease',
+            "keyPress",
+            "keyClick",
+            "keyClicks",
+            "keyEvent",
+            "keyPress",
+            "keyRelease",
+            "keyToAscii",
+            "mouseClick",
+            "mouseDClick",
+            "mouseMove",
+            "mousePress",
+            "mouseRelease",
         ]
         for method_name in method_names:
             method = create_qtest_proxy_method(method_name)
@@ -564,13 +651,14 @@ class QtBot(object):
 QtBot.SignalTimeoutError = SignalTimeoutError
 QtBot.SignalEmittedError = SignalEmittedError
 QtBot.TimeoutError = TimeoutError
+QtBot.CallbackCalledTwiceError = CallbackCalledTwiceError
 
 
 def _add_widget(item, widget):
     """
     Register a widget into the given pytest item for later closing.
     """
-    qt_widgets = getattr(item, 'qt_widgets', [])
+    qt_widgets = getattr(item, "qt_widgets", [])
     qt_widgets.append(weakref.ref(widget))
     item.qt_widgets = qt_widgets
 
@@ -579,7 +667,7 @@ def _close_widgets(item):
     """
     Close all widgets registered in the pytest item.
     """
-    widgets = getattr(item, 'qt_widgets', None)
+    widgets = getattr(item, "qt_widgets", None)
     if widgets:
         for w in item.qt_widgets:
             w = w()
@@ -593,7 +681,7 @@ def _iter_widgets(item):
     """
     Iterates over widgets registered in the given pytest item.
     """
-    return iter(getattr(item, 'qt_widgets', []))
+    return iter(getattr(item, "qt_widgets", []))
 
 
 class _WaitWidgetContextManager(object):
@@ -615,8 +703,8 @@ class _WaitWidgetContextManager(object):
 
     def __enter__(self):
         __tracebackhide__ = True
-        if qt_api.pytest_qt_api != 'pyqt5':
-            raise RuntimeError('Available in PyQt5 only')
+        if qt_api.pytest_qt_api != "pyqt5":
+            raise RuntimeError("Available in PyQt5 only")
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
@@ -626,8 +714,9 @@ class _WaitWidgetContextManager(object):
                 method = getattr(qt_api.QtTest.QTest, self._method_name)
                 r = method(self._widget, self._timeout)
                 if not r:
-                    msg = 'widget {} not {} in {} ms.'.format(self._widget, self._adjective_name, self._timeout)
+                    msg = "widget {} not {} in {} ms.".format(
+                        self._widget, self._adjective_name, self._timeout
+                    )
                     raise TimeoutError(msg)
         finally:
             self._widget = None
-
