@@ -24,7 +24,7 @@ class _AbstractSignalBlocker(object):
         self.raising = raising
         self._signals = None  # will be initialized by inheriting implementations
         self._timeout_message = ""
-        if timeout is None:
+        if timeout is None or timeout == 0:
             self._timer = None
         else:
             self._timer = qt_api.QtCore.QTimer(self._loop)
@@ -46,7 +46,10 @@ class _AbstractSignalBlocker(object):
         if self._timer is not None:
             self._timer.timeout.connect(self._quit_loop_by_timeout)
             self._timer.start()
-        self._loop.exec_()
+
+        if self.timeout != 0:
+            self._loop.exec_()
+
         if not self.signal_triggered and self.raising:
             raise TimeoutError(self._timeout_message)
 
@@ -71,22 +74,28 @@ class _AbstractSignalBlocker(object):
     def _extract_pyqt_signal_name(self, potential_pyqt_signal):
         signal_name = potential_pyqt_signal.signal  # type: str
         if not isinstance(signal_name, str):
-            raise TypeError("Invalid 'signal' attribute in {}. "
-                            "Expected str but got {}".format(signal_name, type(signal_name)))
+            raise TypeError(
+                "Invalid 'signal' attribute in {}. "
+                "Expected str but got {}".format(signal_name, type(signal_name))
+            )
         # strip magic number "2" that PyQt prepends to the signal names
-        signal_name = signal_name.lstrip('2')
+        signal_name = signal_name.lstrip("2")
         return signal_name
 
     def _extract_signal_from_signal_tuple(self, potential_signal_tuple):
         if isinstance(potential_signal_tuple, tuple):
             if len(potential_signal_tuple) != 2:
-                raise ValueError("Signal tuple must have length of 2 (first element is the signal, "
-                                 "the second element is the signal's name).")
+                raise ValueError(
+                    "Signal tuple must have length of 2 (first element is the signal, "
+                    "the second element is the signal's name)."
+                )
             signal_tuple = potential_signal_tuple
             signal_name = signal_tuple[1]
             if not isinstance(signal_name, str):
-                raise TypeError("Invalid type for provided signal name, "
-                                "expected str but got {}".format(type(signal_name)))
+                raise TypeError(
+                    "Invalid type for provided signal name, "
+                    "expected str but got {}".format(type(signal_name))
+                )
             if not signal_name:
                 raise ValueError("The provided signal name may not be empty")
             return signal_name
@@ -119,7 +128,9 @@ class _AbstractSignalBlocker(object):
             name = callback.__name__
         except AttributeError:
             try:
-                name = callback.func.__name__  # e.g. for callbacks wrapped with functools.partial()
+                name = (
+                    callback.func.__name__
+                )  # e.g. for callbacks wrapped with functools.partial()
             except AttributeError:
                 name = ""
         return name
@@ -157,7 +168,7 @@ class SignalBlocker(_AbstractSignalBlocker):
 
         .. note:: contrary to the parameter of same name in
             :meth:`pytestqt.qtbot.QtBot.waitSignal`, this parameter does not
-            consider the :ref:`qt_wait_signal_raising`.
+            consider the :ref:`qt_default_raising` option.
 
     :ivar list args:
         The arguments which were emitted by the signal, or None if the signal
@@ -230,14 +241,20 @@ class SignalBlocker(_AbstractSignalBlocker):
 
     def _get_timeout_error_message(self):
         if self.check_params_callback is not None:
-            return ("Signal {signal_name} emitted with parameters {params} "
-                    "within {timeout} ms, but did not satisfy "
-                    "the {cb_name} callback").format(signal_name=self.signal_name, params=self.get_params_as_str(),
-                                                     timeout=self.timeout,
-                                                     cb_name=self.get_callback_name(self.check_params_callback))
+            return (
+                "Signal {signal_name} emitted with parameters {params} "
+                "within {timeout} ms, but did not satisfy "
+                "the {cb_name} callback"
+            ).format(
+                signal_name=self.signal_name,
+                params=self.get_params_as_str(),
+                timeout=self.timeout,
+                cb_name=self.get_callback_name(self.check_params_callback),
+            )
         else:
-            return "Signal {signal_name} not emitted after {timeout} ms".format(signal_name=self.signal_name,
-                                                                                timeout=self.timeout)
+            return "Signal {signal_name} not emitted after {timeout} ms".format(
+                signal_name=self.signal_name, timeout=self.timeout
+            )
 
 
 class SignalAndArgs:
@@ -251,7 +268,7 @@ class SignalAndArgs:
         # remove signal parameter signature, e.g. turn "some_signal(QString,int)" to "some_signal", because we're adding
         # the actual parameters anyways
         signal_name = self.signal_name
-        signal_name = signal_name.partition('(')[0]
+        signal_name = signal_name.partition("(")[0]
 
         return signal_name + args
 
@@ -266,7 +283,8 @@ class SignalAndArgs:
 
 
 # Returns e.g. "3rd" for 3, or "21st" for 21
-get_ordinal_str = lambda n: "%d%s" % (n, {1: "st", 2: "nd", 3: "rd"}.get(n if n < 20 else n % 10, "th"))
+def get_ordinal_str(n):
+    return "%d%s" % (n, {1: "st", 2: "nd", 3: "rd"}.get(n if n < 20 else n % 10, "th"))
 
 
 class NoMatchingIndexFoundError(Exception):
@@ -324,8 +342,12 @@ class MultiSignalBlocker(_AbstractSignalBlocker):
     def _determine_unique_signals(self, signals):
         # create a map that maps from a unique signal to a list of indices
         # (positions) where this signal is expected (in case order matters)
-        signals_as_str = [str(self.get_signal_from_potential_signal_tuple(signal)) for signal in signals]
-        signal_str_to_unique_signal = {}  # maps from a signal-string to one of the signal instances (the first one found)
+        signals_as_str = [
+            str(self.get_signal_from_potential_signal_tuple(signal))
+            for signal in signals
+        ]
+        # maps from a signal-string to one of the signal instances (the first one found)
+        signal_str_to_unique_signal = {}
         for index, signal_str in enumerate(signals_as_str):
             signal = self.get_signal_from_potential_signal_tuple(signals[index])
             potential_tuple = signals[index]
@@ -344,7 +366,9 @@ class MultiSignalBlocker(_AbstractSignalBlocker):
     def _determine_and_save_signal_name(self, unique_signal_tuple):
         signal_name = self.determine_signal_name(unique_signal_tuple)
         if signal_name:  # might be an empty string if no name could be determined
-            unique_signal = self.get_signal_from_potential_signal_tuple(unique_signal_tuple)
+            unique_signal = self.get_signal_from_potential_signal_tuple(
+                unique_signal_tuple
+            )
             self._signal_names[unique_signal] = signal_name
 
     def _create_signal_emitted_indices(self, signals):
@@ -379,7 +403,8 @@ class MultiSignalBlocker(_AbstractSignalBlocker):
     def _record_emitted_signal_if_possible(self, unique_signal, *args):
         if self._are_signal_names_available():
             self.all_signals_and_args.append(
-                SignalAndArgs(signal_name=self._signal_names[unique_signal], args=args))
+                SignalAndArgs(signal_name=self._signal_names[unique_signal], args=args)
+            )
 
     def _check_signal_match(self, unique_signal, *args):
         if self._order == "none":
@@ -396,15 +421,20 @@ class MultiSignalBlocker(_AbstractSignalBlocker):
         else:  # self.order == "strict"
             if not self._strict_order_violated:
                 # only do the check if the strict order has not been violated yet
-                self._strict_order_violated = True  # assume the order has been violated this time
+                self._strict_order_violated = (
+                    True
+                )  # assume the order has been violated this time
                 if self._check_signal_matches_expected_index(unique_signal, *args):
                     self._signals_emitted[self._signal_expected_index] = True
                     self._signal_expected_index += 1
-                    self._strict_order_violated = False  # order has not been violated after all!
+                    self._strict_order_violated = (
+                        False
+                    )  # order has not been violated after all!
                 else:
                     if self._are_signal_names_available():
                         self._actual_signal_and_args_at_violation = SignalAndArgs(
-                            signal_name=self._signal_names[unique_signal], args=args)
+                            signal_name=self._signal_names[unique_signal], args=args
+                        )
 
     def _all_signals_emitted(self):
         return not self._strict_order_violated and all(self._signals_emitted)
@@ -427,7 +457,9 @@ class MultiSignalBlocker(_AbstractSignalBlocker):
         potential_indices = self._get_unemitted_signal_indices(unique_signal)
         if potential_indices:
             if self._signal_expected_index == potential_indices[0]:
-                if not self._violates_callback_at_index(self._signal_expected_index, *args):
+                if not self._violates_callback_at_index(
+                    self._signal_expected_index, *args
+                ):
                     return True
         return False
 
@@ -446,7 +478,11 @@ class MultiSignalBlocker(_AbstractSignalBlocker):
 
     def _get_unemitted_signal_indices(self, signal):
         """Returns the indices for the provided signal for which NO signal instance has been emitted yet."""
-        return [index for index in self._signals_map[signal] if self._signals_emitted[index] == False]
+        return [
+            index
+            for index in self._signals_map[signal]
+            if not self._signals_emitted[index]
+        ]
 
     def _are_signal_names_available(self):
         if self._signal_names:
@@ -456,9 +492,11 @@ class MultiSignalBlocker(_AbstractSignalBlocker):
     def _get_degenerate_error_message(self):
         received_signals = sum(self._signals_emitted)
         total_signals = len(self._signals_emitted)
-        return ("Received {actual} of the {total} expected signals. "
-                "To improve this error message, provide the names of the signals "
-                "in the waitSignals() call.").format(actual=received_signals, total=total_signals)
+        return (
+            "Received {actual} of the {total} expected signals. "
+            "To improve this error message, provide the names of the signals "
+            "in the waitSignals() call."
+        ).format(actual=received_signals, total=total_signals)
 
     def _get_expected_and_actual_signals_message(self):
         if not self.all_signals_and_args:
@@ -469,25 +507,39 @@ class MultiSignalBlocker(_AbstractSignalBlocker):
 
         missing_signal_strings = []
         for missing_signal_index in self._get_missing_signal_indices():
-            missing_signal_strings.append(self._get_signal_string_representation_for_index(missing_signal_index))
+            missing_signal_strings.append(
+                self._get_signal_string_representation_for_index(missing_signal_index)
+            )
         missing_signals = self._format_as_array(missing_signal_strings)
 
-        return "Emitted signals: {}. Missing: {}".format(emitted_signals, missing_signals)
+        return "Emitted signals: {}. Missing: {}".format(
+            emitted_signals, missing_signals
+        )
 
     @staticmethod
     def _format_as_array(list_of_strings):
-        return "[{}]".format(', '.join(list_of_strings))
+        return "[{}]".format(", ".join(list_of_strings))
 
     def _get_order_violation_message(self):
-        expected_signal_as_str = self._get_signal_string_representation_for_index(self._signal_expected_index)
+        expected_signal_as_str = self._get_signal_string_representation_for_index(
+            self._signal_expected_index
+        )
         actual_signal_as_str = str(self._actual_signal_and_args_at_violation)
-        return ("Signal order violated! Expected {expected} as {ordinal} signal, "
-                "but received {actual} instead. ").format(expected=expected_signal_as_str,
-                                                          ordinal=get_ordinal_str(self._signal_expected_index + 1),
-                                                          actual=actual_signal_as_str)
+        return (
+            "Signal order violated! Expected {expected} as {ordinal} signal, "
+            "but received {actual} instead. "
+        ).format(
+            expected=expected_signal_as_str,
+            ordinal=get_ordinal_str(self._signal_expected_index + 1),
+            actual=actual_signal_as_str,
+        )
 
     def _get_missing_signal_indices(self):
-        return [index for index, value in enumerate(self._signals_emitted) if not self._signals_emitted[index]]
+        return [
+            index
+            for index, value in enumerate(self._signals_emitted)
+            if not self._signals_emitted[index]
+        ]
 
     def _get_signal_string_representation_for_index(self, index):
         """Returns something like <name_of_signal> or <name_of_signal> (callback: <name_of_callback>)"""
@@ -546,12 +598,114 @@ class SignalEmittedSpy(object):
     def assert_not_emitted(self):
         if self.emitted:
             if self.args:
-                raise SignalEmittedError("Signal %r unexpectedly emitted with "
-                                         "arguments %r" %
-                                         (self.signal, list(self.args)))
+                raise SignalEmittedError(
+                    "Signal %r unexpectedly emitted with "
+                    "arguments %r" % (self.signal, list(self.args))
+                )
             else:
-                raise SignalEmittedError("Signal %r unexpectedly emitted" %
-                                         (self.signal,))
+                raise SignalEmittedError(
+                    "Signal %r unexpectedly emitted" % (self.signal,)
+                )
+
+
+class CallbackBlocker(object):
+
+    """
+    .. versionadded:: 3.1
+
+    An object which checks if the returned callback gets called.
+
+    Intended to be used as a context manager.
+
+    :ivar int timeout: maximum time to wait for the callback to be called.
+
+    :ivar bool raising:
+        If :class:`TimeoutError` should be raised if a timeout occured.
+
+        .. note:: contrary to the parameter of same name in
+            :meth:`pytestqt.qtbot.QtBot.waitCallback`, this parameter does not
+            consider the :ref:`qt_default_raising` option.
+
+    :ivar list args:
+        The arguments with which the callback was called, or None if the
+        callback wasn't called at all.
+
+    :ivar dict kwargs:
+        The keyword arguments with which the callback was called, or None if
+        the callback wasn't called at all.
+    """
+
+    def __init__(self, timeout=1000, raising=True):
+        self.timeout = timeout
+        self.raising = raising
+        self.args = None
+        self.kwargs = None
+        self.called = False
+        self._loop = qt_api.QtCore.QEventLoop()
+        if timeout is None:
+            self._timer = None
+        else:
+            self._timer = qt_api.QtCore.QTimer(self._loop)
+            self._timer.setSingleShot(True)
+            self._timer.setInterval(timeout)
+
+    def wait(self):
+        """
+        Waits until either the returned callback is called or timeout is
+        reached.
+        """
+        __tracebackhide__ = True
+        if self.called:
+            return
+        if self._timer is not None:
+            self._timer.timeout.connect(self._quit_loop_by_timeout)
+            self._timer.start()
+        self._loop.exec_()
+        if not self.called and self.raising:
+            raise TimeoutError("Callback wasn't called after %sms." % self.timeout)
+
+    def assert_called_with(self, *args, **kwargs):
+        """
+        Check that the callback was called with the same arguments as this
+        function.
+        """
+        assert self.called
+        assert self.args == list(args)
+        assert self.kwargs == kwargs
+
+    def _quit_loop_by_timeout(self):
+        try:
+            self._cleanup()
+        finally:
+            self._loop.quit()
+
+    def _cleanup(self):
+        if self._timer is not None:
+            _silent_disconnect(self._timer.timeout, self._quit_loop_by_timeout)
+            self._timer.stop()
+            self._timer = None
+
+    def __call__(self, *args, **kwargs):
+        # Not inside the try: block, as if self.called is True, we did quit the
+        # loop already.
+        if self.called:
+            raise CallbackCalledTwiceError("Callback called twice")
+        try:
+            self.args = list(args)
+            self.kwargs = kwargs
+            self.called = True
+            self._cleanup()
+        finally:
+            self._loop.quit()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        __tracebackhide__ = True
+        if value is None:
+            # only wait if no exception happened inside the "with" block
+            self.wait()
 
 
 class SignalEmittedError(Exception):
@@ -561,6 +715,18 @@ class SignalEmittedError(Exception):
     The exception thrown by :meth:`pytestqt.qtbot.QtBot.assertNotEmitted` if a
     signal was emitted unexpectedly.
     """
+
+    pass
+
+
+class CallbackCalledTwiceError(Exception):
+    """
+    .. versionadded:: 3.1
+
+    The exception thrown by :meth:`pytestqt.qtbot.QtBot.waitCallback` if a
+    callback was called twice.
+    """
+
     pass
 
 
