@@ -4,11 +4,9 @@ import pytest
 
 from pytestqt.qt_compat import qt_api
 
-pytestmark = pytest.mark.skipif(qt_api.pytest_qt_api == 'pyside2', reason="https://bugreports.qt.io/browse/PYSIDE-435")
 
-
-@pytest.mark.parametrize('test_succeeds', [True, False])
-@pytest.mark.parametrize('qt_log', [True, False])
+@pytest.mark.parametrize("test_succeeds", [True, False])
+@pytest.mark.parametrize("qt_log", [True, False])
 def test_basic_logging(testdir, test_succeeds, qt_log):
     """
     Test Qt logging capture output.
@@ -33,45 +31,78 @@ def test_basic_logging(testdir, test_succeeds, qt_log):
             qt_api.qInstallMsgHandler(print_msg)
 
         def test_types():
+            # qInfo is not exposed by the bindings yet (#225)
+            # qt_api.qInfo('this is an INFO message')
             qt_api.qDebug('this is a DEBUG message')
             qt_api.qWarning('this is a WARNING message')
             qt_api.qCritical('this is a CRITICAL message')
-            assert {0}
-        """.format(test_succeeds)
+            assert {}
+        """.format(
+            test_succeeds
+        )
     )
-    res = testdir.runpytest(*(['--no-qt-log'] if not qt_log else []))
+    res = testdir.runpytest(*(["--no-qt-log"] if not qt_log else []))
     if test_succeeds:
-        assert 'Captured Qt messages' not in res.stdout.str()
-        assert 'Captured stderr call' not in res.stdout.str()
+        assert "Captured Qt messages" not in res.stdout.str()
+        assert "Captured stderr call" not in res.stdout.str()
     else:
         if qt_log:
-            res.stdout.fnmatch_lines([
-                '*-- Captured Qt messages --*',
-                '*QtDebugMsg: this is a DEBUG message*',
-                '*QtWarningMsg: this is a WARNING message*',
-                '*QtCriticalMsg: this is a CRITICAL message*',
-            ])
+            res.stdout.fnmatch_lines(
+                [
+                    "*-- Captured Qt messages --*",
+                    # qInfo is not exposed by the bindings yet (#232)
+                    # '*QtInfoMsg: this is an INFO message*',
+                    "*QtDebugMsg: this is a DEBUG message*",
+                    "*QtWarningMsg: this is a WARNING message*",
+                    "*QtCriticalMsg: this is a CRITICAL message*",
+                ]
+            )
         else:
-            res.stdout.fnmatch_lines([
-                '*-- Captured stderr call --*',
-                'this is a DEBUG message*',
-                'this is a WARNING message*',
-                'this is a CRITICAL message*',
-            ])
+            res.stdout.fnmatch_lines(
+                [
+                    "*-- Captured stderr call --*",
+                    # qInfo is not exposed by the bindings yet (#232)
+                    # '*QtInfoMsg: this is an INFO message*',
+                    # 'this is an INFO message*',
+                    "this is a DEBUG message*",
+                    "this is a WARNING message*",
+                    "this is a CRITICAL message*",
+                ]
+            )
+
+
+def test_qinfo(qtlog):
+    """Test INFO messages when we have means to do so. Should be temporary until bindings
+    catch up and expose qInfo (or at least QMessageLogger), then we should update
+    the other logging tests properly. #232
+    """
+    if qt_api.pytest_qt_api.startswith("pyside"):
+        assert (
+            qt_api.qInfo is None
+        ), "pyside does not expose qInfo. If it does, update this test."
+        return
+
+    if qt_api.pytest_qt_api.startswith("pyqt4"):
+        pytest.skip("qInfo and QtInfoMsg not supported in PyQt 4")
+
+    qt_api.qInfo("this is an INFO message")
+    records = [(m.type, m.message.strip()) for m in qtlog.records]
+    assert records == [(qt_api.QtInfoMsg, "this is an INFO message")]
 
 
 def test_qtlog_fixture(qtlog):
     """
     Test qtlog fixture.
     """
-    qt_api.qDebug('this is a DEBUG message')
-    qt_api.qWarning('this is a WARNING message')
-    qt_api.qCritical('this is a CRITICAL message')
+    # qInfo is not exposed by the bindings yet (#232)
+    qt_api.qDebug("this is a DEBUG message")
+    qt_api.qWarning("this is a WARNING message")
+    qt_api.qCritical("this is a CRITICAL message")
     records = [(m.type, m.message.strip()) for m in qtlog.records]
     assert records == [
-        (qt_api.QtDebugMsg, 'this is a DEBUG message'),
-        (qt_api.QtWarningMsg, 'this is a WARNING message'),
-        (qt_api.QtCriticalMsg, 'this is a CRITICAL message'),
+        (qt_api.QtDebugMsg, "this is a DEBUG message"),
+        (qt_api.QtWarningMsg, "this is a WARNING message"),
+        (qt_api.QtCriticalMsg, "this is a CRITICAL message"),
     ]
     # `records` attribute is read-only
     with pytest.raises(AttributeError):
@@ -94,11 +125,11 @@ def test_fixture_with_logging_disabled(testdir):
             assert qtlog.records == []
         """
     )
-    res = testdir.runpytest('--no-qt-log')
-    res.stdout.fnmatch_lines('*1 passed*')
+    res = testdir.runpytest("--no-qt-log")
+    res.stdout.fnmatch_lines("*1 passed*")
 
 
-@pytest.mark.parametrize('use_context_manager', [True, False])
+@pytest.mark.parametrize("use_context_manager", [True, False])
 def test_disable_qtlog_context_manager(testdir, use_context_manager):
     """
     Test qtlog.disabled() context manager.
@@ -113,9 +144,9 @@ def test_disable_qtlog_context_manager(testdir, use_context_manager):
     )
 
     if use_context_manager:
-        code = 'with qtlog.disabled():'
+        code = "with qtlog.disabled():"
     else:
-        code = 'if 1:'
+        code = "if 1:"
 
     testdir.makepyfile(
         """
@@ -123,14 +154,16 @@ def test_disable_qtlog_context_manager(testdir, use_context_manager):
         def test_1(qtlog):
             {code}
                 qt_api.qCritical('message')
-        """.format(code=code)
+        """.format(
+            code=code
+        )
     )
     res = testdir.inline_run()
     passed = 1 if use_context_manager else 0
     res.assertoutcome(passed=passed, failed=int(not passed))
 
 
-@pytest.mark.parametrize('use_mark', [True, False])
+@pytest.mark.parametrize("use_mark", [True, False])
 def test_disable_qtlog_mark(testdir, use_mark):
     """
     Test mark which disables logging capture for a test.
@@ -143,7 +176,7 @@ def test_disable_qtlog_mark(testdir, use_mark):
         qt_log_level_fail = CRITICAL
         """
     )
-    mark = '@pytest.mark.no_qt_log' if use_mark else ''
+    mark = "@pytest.mark.no_qt_log" if use_mark else ""
 
     testdir.makepyfile(
         """
@@ -152,7 +185,9 @@ def test_disable_qtlog_mark(testdir, use_mark):
         {mark}
         def test_1():
             qt_api.qCritical('message')
-        """.format(mark=mark)
+        """.format(
+            mark=mark
+        )
     )
     res = testdir.inline_run()
     passed = 1 if use_mark else 0
@@ -173,19 +208,20 @@ def test_logging_formatting(testdir):
             assert 0
         """
     )
-    f = '{rec.type_name} {rec.log_type_name} {rec.when:%Y-%m-%d}: {rec.message}'
-    res = testdir.runpytest('--qt-log-format={0}'.format(f))
-    today = '{0:%Y-%m-%d}'.format(datetime.datetime.now())
-    res.stdout.fnmatch_lines([
-        '*-- Captured Qt messages --*',
-        'QtWarningMsg WARNING {0}: this is a WARNING message*'.format(today),
-    ])
+    f = "{rec.type_name} {rec.log_type_name} {rec.when:%Y-%m-%d}: {rec.message}"
+    res = testdir.runpytest("--qt-log-format={}".format(f))
+    today = "{:%Y-%m-%d}".format(datetime.datetime.now())
+    res.stdout.fnmatch_lines(
+        [
+            "*-- Captured Qt messages --*",
+            "QtWarningMsg WARNING {}: this is a WARNING message*".format(today),
+        ]
+    )
 
 
-@pytest.mark.parametrize('level, expect_passes',
-                         [('DEBUG', 1), ('WARNING', 2), ('CRITICAL', 3),
-                          ('NO', 4)],
-                         )
+@pytest.mark.parametrize(
+    "level, expect_passes", [("DEBUG", 1), ("WARNING", 2), ("CRITICAL", 3), ("NO", 4)]
+)
 def test_logging_fails_tests(testdir, level, expect_passes):
     """
     Test qt_log_level_fail ini option.
@@ -196,7 +232,9 @@ def test_logging_fails_tests(testdir, level, expect_passes):
         """
         [pytest]
         qt_log_level_fail = {level}
-        """.format(level=level)
+        """.format(
+            level=level
+        )
     )
     testdir.makepyfile(
         """
@@ -213,13 +251,16 @@ def test_logging_fails_tests(testdir, level, expect_passes):
     )
     res = testdir.runpytest()
     lines = []
-    if level != 'NO':
-        lines.extend([
-            '*Failure: Qt messages with level {0} or above emitted*'.format(
-                level.upper()),
-            '*-- Captured Qt messages --*',
-        ])
-    lines.append('*{0} passed*'.format(expect_passes))
+    if level != "NO":
+        lines.extend(
+            [
+                "*Failure: Qt messages with level {} or above emitted*".format(
+                    level.upper()
+                ),
+                "*-- Captured Qt messages --*",
+            ]
+        )
+    lines.append("*{} passed*".format(expect_passes))
     res.stdout.fnmatch_lines(lines)
 
 
@@ -284,35 +325,28 @@ def test_logging_fails_ignore(testdir):
     lines = [
         # test1 fails because it has emitted a CRITICAL message and that message
         # does not match any regex in qt_log_ignore
-        '*_ test1 _*',
-        '*Failure: Qt messages with level CRITICAL or above emitted*',
-        '*QtCriticalMsg: a critical message*',
-
+        "*_ test1 _*",
+        "*Failure: Qt messages with level CRITICAL or above emitted*",
+        "*QtCriticalMsg: a critical message*",
         # test2 succeeds because its message matches qt_log_ignore
-
         # test3 fails because of an assert, but the ignored message should
         # still appear in the failure message
-        '*_ test3 _*',
-        '*AssertionError*',
-        '*QtCriticalMsg: WM_DESTROY was sent*(IGNORED)*',
-
+        "*_ test3 _*",
+        "*AssertionError*",
+        "*QtCriticalMsg: WM_DESTROY was sent*(IGNORED)*",
         # test4 fails because one message is ignored but the other isn't
-        '*_ test4 _*',
-        '*Failure: Qt messages with level CRITICAL or above emitted*',
-        '*QtCriticalMsg: WM_PAINT not handled*(IGNORED)*',
-        '*QtCriticalMsg: another critical message*',
-
+        "*_ test4 _*",
+        "*Failure: Qt messages with level CRITICAL or above emitted*",
+        "*QtCriticalMsg: WM_PAINT not handled*(IGNORED)*",
+        "*QtCriticalMsg: another critical message*",
         # summary
-        '*3 failed, 1 passed*',
+        "*3 failed, 1 passed*",
     ]
     res.stdout.fnmatch_lines(lines)
 
 
-@pytest.mark.parametrize('message', ['match-global', 'match-mark'])
-@pytest.mark.parametrize('marker_args', [
-    "'match-mark', extend=True",
-    "'match-mark'"
-])
+@pytest.mark.parametrize("message", ["match-global", "match-mark"])
+@pytest.mark.parametrize("marker_args", ["'match-mark', extend=True", "'match-mark'"])
 def test_logging_mark_with_extend(testdir, message, marker_args):
     """
     Test qt_log_ignore mark with extend=True.
@@ -334,16 +368,17 @@ def test_logging_mark_with_extend(testdir, message, marker_args):
         @pytest.mark.qt_log_ignore({marker_args})
         def test1():
             qt_api.qCritical('{message}')
-        """.format(message=message, marker_args=marker_args)
+        """.format(
+            message=message, marker_args=marker_args
+        )
     )
     res = testdir.inline_run()
     res.assertoutcome(passed=1, failed=0)
 
 
-@pytest.mark.parametrize('message, error_expected', [
-    ('match-global', True),
-    ('match-mark', False),
-])
+@pytest.mark.parametrize(
+    "message, error_expected", [("match-global", True), ("match-mark", False)]
+)
 def test_logging_mark_without_extend(testdir, message, error_expected):
     """
     Test qt_log_ignore mark with extend=False.
@@ -365,7 +400,9 @@ def test_logging_mark_without_extend(testdir, message, error_expected):
         @pytest.mark.qt_log_ignore('match-mark', extend=False)
         def test1():
             qt_api.qCritical('{message}')
-        """.format(message=message)
+        """.format(
+            message=message
+        )
     )
     res = testdir.inline_run()
 
@@ -392,18 +429,17 @@ def test_logging_mark_with_invalid_argument(testdir):
     )
     res = testdir.runpytest()
     lines = [
-        '*= ERRORS =*',
-        '*_ ERROR at setup of test1 _*',
+        "*= ERRORS =*",
+        "*_ ERROR at setup of test1 _*",
         "*ValueError: Invalid keyword arguments in {'does_not_exist': True} "
-            "for qt_log_ignore mark.",
-
+        "for qt_log_ignore mark.",
         # summary
-        '*= 1 error in*',
+        "*= 1 error in*",
     ]
     res.stdout.fnmatch_lines(lines)
 
 
-@pytest.mark.parametrize('apply_mark', [True, False])
+@pytest.mark.parametrize("apply_mark", [True, False])
 def test_logging_fails_ignore_mark_multiple(testdir, apply_mark):
     """
     Make sure qt_log_ignore mark supports multiple arguments.
@@ -413,7 +449,7 @@ def test_logging_fails_ignore_mark_multiple(testdir, apply_mark):
     if apply_mark:
         mark = '@pytest.mark.qt_log_ignore("WM_DESTROY", "WM_PAINT")'
     else:
-        mark = ''
+        mark = ""
     testdir.makepyfile(
         """
         from pytestqt.qt_compat import qt_api
@@ -422,7 +458,9 @@ def test_logging_fails_ignore_mark_multiple(testdir, apply_mark):
         {mark}
         def test1():
             qt_api.qCritical('WM_PAINT was sent')
-        """.format(mark=mark)
+        """.format(
+            mark=mark
+        )
     )
     res = testdir.inline_run()
     passed = 1 if apply_mark else 0
@@ -453,14 +491,16 @@ def test_lineno_failure(testdir):
         """
     )
     res = testdir.runpytest()
-    if qt_api.pytest_qt_api == 'pyqt5':
-        res.stdout.fnmatch_lines([
-            '*test_lineno_failure.py:2: Failure*',
-            '*test_lineno_failure.py:foo:5:*',
-            '    QtWarningMsg: this is a WARNING message',
-        ])
+    if qt_api.pytest_qt_api == "pyqt5":
+        res.stdout.fnmatch_lines(
+            [
+                "*test_lineno_failure.py:2: Failure*",
+                "*test_lineno_failure.py:foo:5:*",
+                "    QtWarningMsg: this is a WARNING message",
+            ]
+        )
     else:
-        res.stdout.fnmatch_lines('*test_lineno_failure.py:2: Failure*')
+        res.stdout.fnmatch_lines("*test_lineno_failure.py:2: Failure*")
 
 
 def test_context_none(testdir):
@@ -468,29 +508,28 @@ def test_context_none(testdir):
     Sometimes PyQt5 will emit a context with some/all attributes set as None
     instead of appropriate file, function and line number.
 
-    Test that when this happens the plugin doesn't break.
+    Test that when this happens the plugin doesn't break, and it filters
+    out the context information.
 
     :type testdir: _pytest.pytester.TmpTestdir
     """
-    if qt_api.pytest_qt_api != 'pyqt5':
-        pytest.skip('Context information only available in PyQt5')
+    if qt_api.pytest_qt_api != "pyqt5":
+        pytest.skip("Context information only available in PyQt5")
     testdir.makepyfile(
         """
         from pytestqt.qt_compat import qt_api
 
         def test_foo(request):
             log_capture = request.node.qt_log_capture
-            context = log_capture._Context(None, None, None)
+            context = log_capture._Context(None, None, 0, None)
             log_capture._handle_with_context(qt_api.QtWarningMsg,
                                              context, "WARNING message")
             assert 0
         """
     )
     res = testdir.runpytest()
-    res.stdout.fnmatch_lines([
-        '*None:None:None:*',
-        '* QtWarningMsg: WARNING message*',
-    ])
+    assert "*None:None:0:*" not in str(res.stdout)
+    res.stdout.fnmatch_lines(["* QtWarningMsg: WARNING message*"])
 
 
 def test_logging_broken_makereport(testdir):
@@ -501,7 +540,8 @@ def test_logging_broken_makereport(testdir):
 
     :type testdir: _pytest.pytester.TmpTestdir
     """
-    testdir.makepyfile(conftest="""
+    testdir.makepyfile(
+        conftest="""
         import pytest
 
         @pytest.mark.hookwrapper(tryfirst=True)
@@ -509,7 +549,8 @@ def test_logging_broken_makereport(testdir):
             if call.when == 'call':
                 raise Exception("This should not be hidden")
             yield
-    """)
+    """
+    )
     p = testdir.makepyfile(
         """
         def test_foo():
@@ -517,6 +558,4 @@ def test_logging_broken_makereport(testdir):
         """
     )
     res = testdir.runpytest_subprocess(p)
-    res.stdout.fnmatch_lines([
-        '*This should not be hidden*',
-    ])
+    res.stdout.fnmatch_lines(["*This should not be hidden*"])
